# PARTICLE CONSTANTS
A_PARTICLE = "あ"
E_PARTICLE = "え"
I_PARTICLE = "い"
O_PARTICLE = "お"
U_PARTICLE = "う"

KA_PARTICLE = "か"
KE_PARTICLE = "け"
KI_PARTICLE = "き"
KO_PARTICLE = "こ"
KU_PARTICLE = "く"

GA_PARTICLE = "が"
GE_PARTICLE = "げ"
GI_PARTICLE = "ぎ"
GO_PARTICLE = "ご"
GU_PARTICLE = "ぐ"

SA_PARTICLE = "さ"
SE_PARTICLE = "せ"
SHI_PARTICLE = "し"
SO_PARTICLE = "そ"
SU_PARTICLE = "す"

TA_PARTICLE = "た"
TE_PARTICLE = "て"
CHI_PARTICLE = "ち"
TO_PARTICLE = "と"
TSU_PARTICLE = "つ"

DA_PARTICLE = "だ"
DE_PARTICLE = "で"
DI_PARTICLE = "ぢ"
DO_PARTICLE = "ど"
DU_PARTICLE = "づ"

NA_PARTICLE = "な"
NE_PARTICLE = "ね"
NI_PARTICLE = "に"
NO_PARTICLE = "の"
NU_PARTICLE = "ぬ"

HA_PARTICLE = "は"
HE_PARTICLE = "へ"
HI_PARTICLE = "ひ"
HO_PARTICLE = "ほ"
HU_PARTICLE = "ふ"

BA_PARTICLE = "ば"
BE_PARTICLE = "べ"
BI_PARTICLE = "び"
BO_PARTICLE = "ぼ"
BU_PARTICLE = "ぶ"

PA_PARTICLE = "ぱ"
PE_PARTICLE = "ぺ"
PI_PARTICLE = "ぴ"
PO_PARTICLE = "ぽ"
PU_PARTICLE = "ぷ"

MA_PARTICLE = "ま"
ME_PARTICLE = "め"
MI_PARTICLE = "み"
MO_PARTICLE = "も"
MU_PARTICLE = "む"

YA_PARTICLE = "や"
YU_PARTICLE = "ゆ"
YO_PARTICLE = "よ"

RA_PARTICLE = "ら"
RE_PARTICLE = "れ"
RI_PARTICLE = ""
RO_PARTICLE = "ろ"
RU_PARTICLE = "る"

WA_PARTICLE = "わ"
WO_PARTICLE = "を"
N_PARTICLE = "ん"

CHISAI_TSU_PARTICLE = "っ"
