from .EnumeratedTypes import Formality, Polarity, Tense, VerbClass

KUDASAI = "ください"

# IRREGULAR VERB ENDINGS
SURU_ENDING = "する"
KURU_ENDING = "くる"

# (POLITE) MASU FORM ENDINGS
MASU_POSITIVE_NONPAST = "ます"
MASU_NEGATIVE_NONPAST = "ません"
MASU_POSITIVE_PAST = "ました"
MASU_NEGATIVE_PAST = "ませんでした"

# (PLAIN) KATTA FORM ENDING
KATTA_ENDING = "かった"

# (PLAIN) NAI ENDING
NAI_ENDING = "ない"

# VOLITIONAL FORM ENDINGS
VOLITIONAL_POLITE_ENDING = "ましょう"
VOLITIONAL_POLITE_COPULA = "でしょう"
VOLITIONAL_PLAIN_COPULA = "だろう"
VOLITIONAL_ICHIDAN_PLAIN_ENDING = "よう"
VOLITIONAL_SURU_ENDING = "しよう"
VOLITIONAL_KURU_ENDING = "こよう"

# POTENTIAL FORM
POTENTIAL_POLITE_ICHIDAN_ENDING = "られます"
POTENTIAL_ICHIDAN_ENDING = "られる"
POTENTIAL_SURU_PLAIN_POSITIVE_ENDING = "できる"
POTENTIAL_SURU_PLAIN_NEGATIVE_ENDING = "できない"
POTENTIAL_SURU_POLITE_POSITIVE_ENDING = "できます"
POTENTIAL_SURU_POLITE_NEGATIVE_ENDING = "できません"
POTENTIAL_KURU_PLAIN_POSITIVE_ENDING = "こられる"
POTENTIAL_KURU_POLITE_POSITIVE_ENDING = "こられます"
POTENTIAL_KURU_PLAIN_NEGATIVE_ENDING = "こられない"
POTENTIAL_KURU_POLITE_NEGATIVE_ENDING = "こられません"

# PASSIVE FORM ENDINGS
PASSIVE_ICHIDAN_PLAIN_POSITIVE_ENDING = "られる"
PASSIVE_ICHIDAN_POLITE_POSITIVE_ENDING = "られます"
PASSIVE_ICHIDAN_PLAIN_NEGATIVE_ENDING = "られない"
PASSIVE_ICHIDAN_POLITE_NEGATIVE_ENDING = "られません"

# CAUSATIVE VERB ENDINGS
CAUSATIVE_PLAIN_SURU_ENDING = "させる"
CAUSATIVE_PLAIN_KURU_ENDING = "こさせる"
CAUSATIVE_KURU_NEGATIVE_BASE = "こさせ"

# IMPERATIVE VERB ENDINGS
IMPERATIVE_SURU_PLAIN_POSITIVE_ENDING = "しろ"
IMPERATIVE_KURU_PLAIN_POSITIVE_ENDING = "こい"

# PROVISIONAL VERB ENDINGS
PROVISIONAL_SURU_PLAIN_POSITIVE_ENDING = "すれば"
PROVISIONAL_SURU_POLITE_POSITIVE_ENDING = "しませば"
PROVISIONAL_KURU_PLAIN_POSITIVE_ENDING = "くれば"
PROVISIONAL_KURU_POLITE_POSITIVE_ENDING = "きませば"
PROVISIONAL_ICHIDAN_PLAIN_NEGATIVE_ENDING = "なければ"

# PASSIVE VERB ENDINGS
PASSIVE_SURU_PLAIN_POSITIVE_ENDING = "される"
PASSIVE_KURU_PLAIN_POSITIVE_ENDING = "こられる"
